/* simpleFFTfilter.c

Simpler FFT-Filter mit Glttung,
bentigt GNU GSL

compile with: gcc -lm -lgsl -lgslcblas -O simpleFFTfilter.c -o simpleFFTfilter

Aufruf: simpleFFTfilter Punkte

Nach der Rcktrafo wird zwar ein gleitender Durchschnitt angewendet,
dieser kann aber ohne den FFT-Filter die Interferenzen nicht eliminieren.

(c) Jochen Puchalla 2002
License: GPL

*/

#include <gsl/gsl_fft_real.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>

#define PUNKTE 1048576

double sum,rest;
double ywert[PUNKTE];
double xwert[PUNKTE];
double temp[PUNKTE];
int Anzahl,nix,haelfte,i,j,points,smooth;
size_t schritt=1;
size_t breite; /* muss 2 hoch n sein */

/* gibt 4Term-Blackman-Harris-Wert zurck, input zwischen 0 und 1 */
/* Alternative zur Parabel. */
double blackharr4(double stelle)
{
 double A0=0.35875;
 double A1=0.48829;
 double A2=0.14128;
 double A3=0.01168;
 double BH;
 double pi=3.1415926536;
 BH=A0+A1*cos(pi*stelle)+A2*cos(2*pi*stelle)+A3*cos(3*pi*stelle);
 return BH;
}


/* -x^2+1 */
double parabel(double stelle)
{
 double PARA;
 PARA=(-1)*pow(stelle,2)+1;
 //return PARA;
 return 1; /* Parabel macht Kurve zu weich */
}


/* gibt Zahlenstring Zahl als integer zurck */
int makeint(char *Zahl)
{
 int a=0;int count;
 for(count=0;isdigit(Zahl[count]);count++) a=a*10+Zahl[count]-48;
 return a;
}

/* ###################################################################### */
 
int main(int argc, char *argv[])
{

if (argc<2) {printf("\nusage: simpleFFTfilter points  < inputfile  > outputfile\n\n");exit(1);}
points=makeint(argv[1]);

//printf("Zahlenpaare eingeben: \n");
for (i=0;i<PUNKTE;i++) nix=scanf("%lf%lf", &xwert[i], &ywert[i]);
//for (i=0;i<PUNKTE;i++) if(;

/* Anzahl der Datenpunkte bestimmen */
for (i=10;i<PUNKTE;i++) if (xwert[i]==0){Anzahl=i; i=PUNKTE;}

/* breite bestimmen, mu n^2 sein */
for (i=8;i<Anzahl;i=i*2) breite=i*2; 
//printf("%d",breite);

smooth=breite/points; /* smooth ist Parameter fr Frequenzfilter, points fr Durchschnitt */
//printf("%d\n", smooth);

/* Ausgabe der Eingabe: */
//for (i=0;i<breite;i++) {  printf("%d %f %f \n", i,xwert[i],ywert[i]*2);}


/* TRAFO: */
nix=gsl_fft_real_radix2_transform(ywert,schritt,breite);

/* Ausgabe Trafo-Daten */
//for (i=0;i<breite;i++) printf("%d %f %f \n", i,xwert[i],ywert[i]);



/* ##### FILTER ##### 

/* Parabelabfall: */
for (i=0;i<smooth;i++) {rest=i; ywert[i]=ywert[i]*parabel(i/smooth);}

/* Nullsetzen in der Mitte */
for (i=smooth;i<breite-smooth;i++) ywert[i]=0;

/* Parabelanstieg: */
for (i=breite-smooth;i<breite;i++) {rest=breite-i; ywert[i]=ywert[i]*parabel(rest/smooth);}



/* RCKTRAFO: */
nix=gsl_fft_halfcomplex_radix2_inverse(ywert,schritt,breite);

/* gleitender Durchschnitt: */
haelfte=points/2;
	//printf("%d %d\n",haelfte*2,points);
if(haelfte*2==points) /* gerade */
{
	//printf("gerade\n");
for (i=0;i<breite;i++) {sum=(ywert[i-haelfte]+ywert[i+haelfte])/2; for (j=-haelfte+1;j<=haelfte-1;j++) {sum=sum+ywert[i+j]; temp[i]=sum/points;}}
}
else /* ungerade */
{
	//printf("ungerade\n");
for (i=0;i<breite;i++) {sum=0; for (j=-haelfte;j<=haelfte;j++) {sum=sum+ywert[i+j]; temp[i]=sum/points;}}
}
for (i=0;i<breite;i++) ywert[i]=temp[i];


//printf("Rcktrafo-Daten: \n");
for (i=0;i<breite;i++) {if(xwert[i]!=0 && i!=0)  printf("%f %f \n", xwert[i],ywert[i]);}

}; 
